--[[Script Name: BetterGetTemplates.lua
Author: Theos
Version: 1.0
Public Release: 0.1
Purpose: This modification displays a dynamic window everytime you use the /create command with a partial matching search string.
		 For example /create cooked will open a window containing "animalparts_cooked_meat" and "cooked_staw" the create button
		 will give a cursor for the spawn location
]]--



function GetTemplateMatch(templateSearchStr)
	templateList = GetAllTemplateNames()

	-- if we have an exact match, then return it
	if( IsInTableArray(templateList,templateSearchStr) ) then
		return templateSearchStr
	end

	matches = {}
	for i, templateName in pairs(templateList) do
		if (templateName:find(templateSearchStr) ~= nil) then
			matches[#matches+1] = templateName
		end
	end

	if( #matches == 1 ) then
		return matches[1]
	elseif( #matches > 1 ) then
		resultStr = "Multiple templates match: "
		for i, match in pairs(matches) do
			resultStr = resultStr .. ", " .. match
		end
		--this:SystemMessage(resultStr)
		DoDisplayTemplates(matches)
		return nil
	else
		this:SystemMessage("No template matches search string")
	end
end


curCreateTable = nil
local templatecreateListPage = 1

function DoDisplayTemplates(results)

	if (results ~= nil) then
		curCreateTable = results
	end

	local newWindow = DynamicWindow("CreateTemplateWindow","Found Templates",500,640)

	local y = 10
	local itemsPerPage = 18
	local startIndex = (templatecreateListPage-1) * itemsPerPage
	for i,entry in pairs(curCreateTable) do
		if( i >= startIndex and i <= startIndex + itemsPerPage) then
			local varName = entry
			if( varName:len() > 45 ) then
				varName = varName:sub(1,22).."..."
			end
			newWindow:AddLabel(20, y, varName)
			newWindow:AddButton(350, y, "create:"..varName,"Create", 80, 0, "", "", false)

			y = y + 30
		end
	end

	if( templatecreateListPage > 1 ) then
		newWindow:AddButton(80, 580, "CreateListPrev:CreateListPrev", "Previous", 100, 0, "", "", false)
	end
	if( templatecreateListPage <= #curCreateTable / itemsPerPage) then
		newWindow:AddButton(230, 580, "CreateListNext:CreateListNext", "Next", 100, 0, "", "", false)
	end

	this:OpenDynamicWindow(newWindow)
end


RegisterEventHandler(EventType.DynamicWindowResponse,"CreateTemplateWindow",
	function (user,returnId)
		if(returnId ~= nil) then
			action, template = string.match(returnId, "(%a+):([%a_]+)")

			--DebugMessage(action)

			if(action == "create") then
				templateId = GetTemplateMatch(template)
				if( templateId ~= nil ) then
					this:RequestClientTargetLoc(this,"createTemplateAt")
				end
			elseif(action == "CreateListNext") then
			templatecreateListPage = templatecreateListPage + 1
			DoDisplayTemplates(nil)
			elseif( action == "CreateListPrev") then
				templatecreateListPage = templatecreateListPage - 1
				DoDisplayTemplates(nil)
			end
		end
	end)
