require 'Guild_gumps'

-- Internal structure of the guild
-- the memberinfo table keeps track of the member
-- settings
GuildMemberInfo = {
					Create = function(m)
						local res = {}

						res.AccessLevel="Member";
						res.JoinDate = os.time();
						res.Name = m:GetName();
						res.Kick = false;
						res.LastOnline = os.time();
						return res;
					end
				  }

GuildInvitation = {
					Create = function(m, g)
						local res = {}
							res.InvitedBy = m
							res.Guild = g

						return res;
					end
				  }

-- Creates a random guid as guild id
local random = math.random
local function uuid()
    local template ='xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'
    return string.gsub(template, '[xy]', function (c)
        local v = (c == 'x') and random(0, 0xf) or random(8, 0xb)
        return string.format('%x', v)
    end)
end

-- Main Guild Module
Guild = {
		Capacity = 10,

		InviteTarget = function()
			this:RequestClientTargetGameObj(this, "guildinvite")
		end,
		Get = function (mobile)

			if (mobile == nil) then
				return nil;
			end

			--DebugMessage("Guild: Get");

			local g = mobile:GetObjVar("Guild")

			if (g == nil) then return nil end

			return GetGuildStone():GetObjVar(g)
		end,

		Invite = function ( from, target )
			local g = Guild.Get( from );

			if ( g == nil ) then
				from:SystemMessage("You have not yet created a guild!")
				return
			end

			target:SystemMessage("You are invited to join the guild: "..g.Name)

			from:SystemMessage( "You have invited them to join your guild." );

			target:SetObjVar("GuildInvitation", GuildInvitation.Create(from, g.Id))
		end,

		Decline = function()

		if( this:HasObjVar("GuildInvitation") ) then

			local guildInvite = this:GetObjVar("GuildInvitation")

			if (guildInvite == nil) then
				this:SystemMessage( "Guild: Nobody invited you" );
				return
			end

			local g = Guild.Get(guildInvite.InvitedBy)

			if (g == nil) then
				this:SystemMessage( "The guild no longer Exists" );
				this:SetObjVar("GuildInvitation",nil);
				return
			end

			guildInvite.InvitedBy:SystemMessage( "Does not wish to join your guild." );

			this:SystemMessage( "You notify them that you do not wish to join the guild." );

			this:SetObjVar("GuildInvitation",nil);
		end
	end,

	Accept = function()
			if( this:HasObjVar("GuildInvitation") ) then
				local guildInvitation = this:GetObjVar("GuildInvitation")

				local g =Guild.Get(guildInvitation.InvitedBy)

				if (g == nil) then
					this:SystemMessage( "The guild no longer Exists" );
					this:SetObjVar("GuildInvitation",nil);
					return
				end

				this:SetObjVar("GuildInvitation",nil);


				local mi = g.Members[this.Id];

				if ( mi == nil ) then
					g.Members[this.Id] = GuildMemberInfo.Create( this );
					this:SetObjVar("Guild", g.Id);

					Guild.SendToAll( this, g, "joined the guild.");

					this:SystemMessage( "You have joined the guild" );

				end

				GetGuildStone():SetObjVar(g.Id, g)
			else
				this:SystemMessage( "Accept Guild: Nobody invited you" );
			end
	end,

	Contains = function( m , g )
			return ( g.Members[m.Id] ~= nil );
	end,

	Remove = function ( m, g )
		m:SystemMessage("You have been removed from the guild.");

		Guild.SendToAll( nil, g, "A player has been removed from your guild.");

		if (g.Members[m.Id] ~= nil) then
			g.Members[m.Id] = nil;
			m:SetObjVar("Guild", nil);
		end

		GetGuildStone():SetObjVar(g.Id, g)

		m:SendMessage("UpdateName")

	end,

	SendToAll = function(from, g , line)

		local name = ""
		if ( from ~= nil) then
			local actualName = from:GetObjVar("actualName")

			if (actualName == nil) then
				name = from:GetName()
			else
				name = actualName
			end
		end

		for mobile, memberinfo in pairs(g.Members) do
			local mob = GameObj(mobile)

			if (mob ~= nil) then
				mob:SystemMessage( "[00FF00] [Guild]" .. name ..":-> " .. line.."[-]") ;
			end
		end
	end,

	SendMessage = function (...)

	local g = Guild.Get(this)

		if (g == nil) then
			return
		end

		local arg = table.pack(...)

		local line = ""
		if(#arg > 0) then
			for i = 1,#arg do line = line .. tostring(arg[i]) .. " " end
		end

		Guild.SendToAll( this, g, line);
	end,

	GuildMenu = function()
		GuildInfo()
	end,
	Create = function(leader)
		local res = {}
		res.Leader = leader;

		res.Members = {}

		local mi = GuildMemberInfo.Create( leader );
		mi.AccessLevel = "Leader"

		res.Members[leader.Id] = mi
		res.Id = uuid()
		res.Name = "New Guild"
		res.Renames = 0
		leader:SystemMessage("You created a guild");

		GetGuildStone():SetObjVar(res.Id, res)
		leader:SetObjVar("Guild",res.Id)

		return res
	end,
	Disband = function(user)
		local g = Guild.Get(user)

		if (g == nil) then
			return
		end

		Guild.SendToAll( this, g, "Your guild has been disbanded!");

		for mobile, entry in pairs (g.Members) do
			local mob = GameObj(mobile)

			if (mob ~= nil) then
				mob:SetObjVar("Guild", nil)
			end
		end

		GetGuildStone():SetObjVar(g.Id, nil)
	end,
	Rename = function(...)

		local arg = table.pack(...)

		local line = ""
		if(#arg > 0) then
			for i = 1,#arg do line = line .. tostring(arg[i]) .. " " end
		end

		local g = Guild.Get(this)

		if (g == nil) then return end

		if (g.Renames == nil) then
			g.Renames = 0
		end

		if (g.Renames >3) then
			this:SystemMessage("A guild can only be renamed three times")
			return
		end

		if (string.len(line) < 3) then
			this:SystemMessage("Your guildname must be at least 3 characters long!")
			return
		end

		if (string.len(line) > 20) then
			this:SystemMessage("No more than 20 characters allowed!")
			return
		end

		g.Name = line

		g.Renames = g.Renames + 1

		GetGuildStone():SetObjVar(g.Id, g)

		for mobile, entry in pairs (g.Members) do
			local mob = GameObj(mobile)

			if (mob ~= nil) then
				mob:SendMessage("UpdateName")
			end
		end

		this:SystemMessage("Guild Renamed! ")
	end,
	ValidateInvitation = function(invite)
		if (invite == nil) then
			return nil
		end

		local guild = invite.Guild

		if (guild == nil) then
			return nil
		end

		if (GetGuildStone():HasObjVar(guild)) then
			return GetGuildStone():GetObjVar(guild)
		end

		return nil

	end,
	UpdateMemberInfo = function(user, g)

		--local g = Guild.Get(user)

		if (g == nil) then return end
		DebugMessage("Updating...")

		local mi = g.Members[user.Id];

		if ( mi ~= nil ) then

			DebugMessage("Setting Member info")
			if (mi.Kick == false) then

				mi.LastOnline = os.time()

				local name = ""

				local actualName = user:GetObjVar("actualName")

				if (actualName == nil) then
					name = user:GetName()
				else
					name = actualName
				end

				mi.Name = name

				g.Members[user.Id] = mi

				DebugMessage("writing..."..g.Members[user.Id].Name.." - "..g.Members[user.Id].LastOnline)
			else
				user:SystemMessage("You have been removed from the guild")
				g.Members[user.Id] = nil
				user:SetObjVar("Guild", nul)
			end

			GetGuildStone():SetObjVar(g.Id, g);
		end

	end,

	EditMember = function(mobileId, g)

		DebugMessage("Guild: Got request for "..tostring(mobileId))
		MemberInfoGump(mobileId, g)
	end,
	CheckAccessLevel = function(user,required)

		local g = Guild.Get(user)

		if (g == nil) then
			return
		end

		local mi = g.Members[user.Id]

		local level = mi.AccessLevel

		if (level == "Leader") then return true end

		if (level == "CoLeader" and required == "CoLeader") then return true end
		if (level == "CoLeader" and required == "Emissary") then return true end
		if (level == "CoLeader" and required == "Member") then return true end
		if (level == "CoLeader" and required == "Trial") then return true end

		if (level == "Emissary" and required == "Emissary") then return true end
		if (level == "Emissary" and required == "Member") then return true end
		if (level == "Emissary" and required == "Trial") then return true end

		if (level == "Member" and required == "Member") then return true end
		if (level == "Member" and required == "Trial") then return true end

		if (level == "Trial" and required == "Trial") then return true end

		return false

	end,
	GetAccessLevel = function(id)
		local g = Guild.Get(this)

		if (g == nil) then return nil end

		local mi = g.Members[id]

		if (mi == nil) then return nil end

		return mi.AccessLevel

	end,
	CanBePromotedBy = function(mylevel, theirlevel)
		if (mylevel == "Leader" and theirlevel =="Leader") then return false end
		if (mylevel == "CoLeader" and theirlevel =="Leader") then return false end
		if (mylevel == "CoLeader" and theirlevel =="CoLeader") then return false end

		if (mylevel == "Emissary" and theirlevel =="Leader") then return false end
		if (mylevel == "Emissary" and theirlevel =="CoLeader") then return false end
		if (mylevel == "Emissary" and theirlevel =="Emissary") then return false end

		return true
	end,

	PromoteAccessLevel = function (current)
		if (current == "Leader") then return "Leader" end
		if (current == "CoLeader") then return "Leader" end
		if (current == "Emissary") then return "CoLeader" end
		if (current == "Member") then return "Emissary" end
		if (current == "Trial") then return "Member" end

		return "Member"
	end,
	DemoteAccessLevel = function (current)
		if (current == "Leader") then return "CoLeader" end
		if (current == "CoLeader") then return "Emissary" end
		if (current == "Emissary") then return "Member" end
		if (current == "Member") then return "Trial" end
		if (current == "Trial") then return "Trial" end

		return "Member"
	end,
	PromoteMember = function(id, g)
		if (g == nil or not(Guild.CheckAccessLevel(this,"Emissary"))) then return end

		local mylevel = Guild.GetAccessLevel(this.Id)
		local theirlevel = Guild.GetAccessLevel(id)

		if ( not(Guild.CanBePromotedBy(mylevel, theirlevel)) ) then
			this:SystemMessage("Access level not high enough")
			return
		end

		DebugMessage("Guild: Promoting..."..tostring(id))

		local mi = g.Members[id];

		if ( mi ~= nil ) then
			local currentLevel =  mi.AccessLevel

			mi.AccessLevel = Guild.PromoteAccessLevel(currentLevel)

			g.Members[id] = mi

			DebugMessage("writing..."..g.Members[id].AccessLevel)

			GetGuildStone():SetObjVar(g.Id, g);
		end
	end,
	DemoteMember = function(id, g)
		if (g == nil or not(Guild.CheckAccessLevel(this,"Emissary"))) then return end


		local mylevel = Guild.GetAccessLevel(this.Id)
		local theirlevel = Guild.GetAccessLevel(id)

		if (not(Guild.CanBePromotedBy(mylevel, theirlevel))) then
			this:SystemMessage("Access level not high enough")
			return
		end

		local mi = g.Members[id];

		if ( mi ~= nil ) then
			local currentLevel =  mi.AccessLevel

			mi.AccessLevel = Guild.DemoteAccessLevel(currentLevel)

			g.Members[id] = mi

			DebugMessage("writing..."..g.Members[id].AccessLevel)
			GetGuildStone():SetObjVar(g.Id, g);
		end
	end,
    }

	function split(str, pat)
   local t = {}  -- NOTE: use {n = 0} in Lua-5.0
   local fpat = "(.-)" .. pat
   local last_end = 1
   local s, e, cap = str:find(fpat, 1)
   while s do
      if s ~= 1 or cap ~= "" then
	 table.insert(t,cap)
      end
      last_end = e+1
      s, e, cap = str:find(fpat, last_end)
   end
   if last_end <= #str then
      cap = str:sub(last_end)
      table.insert(t, cap)
   end
   return t
end
