require 'incl_dialogwindow'
require 'incl_party_helpers'
require 'Incl_Party'

-- This function handles the player targeted event to check if all conditions are met
-- After that Party.Invite is called to do the management stuff
RegisterEventHandler(EventType.ClientTargetGameObjResponse, "partyinvite",
	function(target,user)
		Party.Command()

		if( target == nil ) then
			return
		end

		if not( target:IsPlayer() ) then
			user:SystemMessage("Error: You can only invite players")
			return
		end

		--if (target == user) then
		--	user:SystemMessage("You cannot add yourself to a party.")
		--	return
		--end

		local p = Party.Get( user );
		local mp = Party.Get( target );

			if ( p ~= nil and p.Leader ~= user ) then
				user:SystemMessage("You may only add members to the party if you are the leader.")
			return
			end

			if ( p ~= nil and (Party.GetMembers(p) + Party.GetCandidates(p) >= Party.Capacity) ) then
				user:SystemMessage("You may only have 10 in your party (this includes candidates).")
				return
			end

			if ( mp ~= nil and mp == p ) then
				user:SystemMessage("This person is already in your party!")
				return
			end

			if ( mp ~= nil ) then
				user:SystemMessage("This person is already in a party!")
				return
			end

			Party.Invite(user, target)
	end)

-- This event handles a player logging out. If its a regular party member his/her invitation
-- to the party will be saved and issued again after loggin in
RegisterEventHandler(EventType.UserLogout,"",function ()
	--GetPartyController():ScheduleTimerDelay(TimeSpan.FromSeconds(15.0),"PartyLeaveTimer",this)
	DebugMessage("Removing Player from party"..this:GetName())

	local p = Party.Get(this)

	if (p == nil) then
		return
	end

	Party.Remove( this, p )
	Party.ResetPartyTitle(this)

	if (p.Leader ~= this) then
		this:SetObjVar("PartyReInvitation", p.Leader)
	end
end)

-- This event will be called after the user logged in again. If the party
-- the player was previously in still exists an invitation will be sent
-- when there is still a slot available
RegisterEventHandler(EventType.Message,"EnterWorld",function ()
	Party.ResetPartyTitle(this)

	local leader = this:GetObjVar("PartyReInvitation")

	if (leader ~= nil) then
		local p = Party.Get( leader );

		if ( p ~= nil and (Party.GetMembers(p) + Party.GetCandidates(p) < Party.Capacity) ) then
			Party.Invite(leader, this)
			return
		end

		this:SetObjVar("PartyReInvitation", nil);
	end
end)

-- This Event handles the response of the party window
RegisterEventHandler(EventType.DynamicWindowResponse, "PartyWindow",
function (user,buttonId)
	if (user == nil) then return end

	local p = Party.Get(user)

	-- Handles the invite command of the dynamic window
	if (buttonId == "Invite") then
		Party.InviteTarget()
	end

	-- Handles the leave party event
	if (buttonId == "Leave") then
		if (p == nil) then
			user:SystemMessage("You are not in a party");
			return
		end

		Party.Remove(user, p)
	end

	-- Handles the toggle loot button
	if (buttonId == "Loot") then

		if (p == nil) then
				user:SystemMessage("You are not in a party");
				return
			end

		local mi = p.Members[user];

		if ( mi ~= nil ) then
			mi.CanLoot = not mi.CanLoot;

			if ( mi.CanLoot ) then
				user:SystemMessage( "You have chosen to allow your party to loot your corpse." );
			else
				user:SystemMessage( "You have chosen to prevent your party from looting your corpse." );
			end

			GetPartyController():SetObjVar(p.Id, p)
		end
	end

	if (buttonId =="Refresh") then
		Party.Command()
	end

	-- Handles the kick player event (X) buttonId. The prefix
	-- Kick- is used to identify the command. That part is catched by
	-- the match() function. The rest is taken as function parameter gameobj number
	-- and converted back to a game obj
	if (buttonId:match("Kick-") ) then
		if (p == nil) then
			return
		end

		local objVarName = buttonId:sub(6)
		local varnum = tonumber(objVarName)
		local target = GameObj(varnum)

		Party.Remove(target, p)
		Party.Command()
	end

	-- This handles the Dump party buttonId
	-- it opens a window with all available party attached to
	-- the party controller
	if (buttonId == "Dump") then

		local array = {}
		for key, value in pairs(GetPartyController():GetAllObjVars()) do
			table.insert(array,{name = key, value = value})
		end

		if (array == nil) then
			DebugMessage("Warning no Party!");
		end

		local newWindow = DynamicWindow("PartyDumpWindow","Partys",600,640)

		local y = 10
		local itemsPerPage = 18
		local startIndex = (objVarPage-1) * itemsPerPage
		local i = 0
		for id,entry in pairs(array) do
			if( i >= startIndex and i <= startIndex + itemsPerPage) then
				local varName = (entry.name)
				if( varName:len() > 35 ) then
					varName = varName:sub(1,32).."..."
				end

				local numMem = tostring(Party.GetMembers(entry.value))
				local numCand = tostring(Party.GetCandidates(entry.value))

				local name = entry.value.Leader:GetName()

				newWindow:AddLabel(20, y, varName)
				newWindow:AddLabel(370, y, numMem)
				newWindow:AddLabel(385, y, numCand)

				newWindow:AddLabel(400, y, name)

				y = y + 30
				i = i + 1
			end
		end

		this:OpenDynamicWindow(newWindow)
	end
end)

-- this event is run when the player didn't respond to a party invitation
RegisterEventHandler(EventType.Timer,"DeclineTimer",
    function(target)
		this:SystemMessage( "Decline: timer tick" );
		Party.Decline()
    end)
