-- Poison effect (bensku)
require "incl_effect_handlers"
require "incl_effect_utils"

EFFECT_TICK_SPEED = 1000 -- 1000ms = 1s

effects = {} -- DO NOT TOUCH
ordersLocked = false -- DO NOT TOUCH (unless modifying orders directly)
cureOrders = {}
effectOrders = {}
needsTicking = false -- True if effect ticking is needed
cureProcesses = {}
clearFlag = false

function ApplyEffectTick()
	if clearFlag then
		for id,effect in pairs(effects) do
			OrderCure(id)
		end
	end

	--this:SystemMessage("Effect tick start")
	if effects == nil or effects == {} or #effects < 1 then
		--this:SystemMessage("effects variable is empty...")
		if this:HasObjVar("StatusEffects") then
			--this:SystemMessage("...grabbing it from ObjVar")
			effects = this:GetObjVar("StatusEffects")
		end
		if effects == {} or effects == nil then
			effects = {}
			this:DelObjVar("StatusEffects") -- Just in case...
		end
	end

	if not ordersLocked then
		--this:SystemMessage("Orders are not locked")
		ordersLocked = true

		ExecuteCureProcesses()

		local hasCured = false
		for k,v in pairs(cureOrders) do
			hasCured = true
			local handlers = ProduceEffectHandlers(this, effects[v], "OnRemove")
			for k,v in pairs(handlers) do
				v:OnRemove()
			end
			effects[v] = nil
		end

		if hasCured then
			local sparseEffects = effects
			effects = {}
			for oldId,effect in pairs(effects) do
				effects[#effects+1] = effect
			end
		end
		
		cureOrders = {}

		for k,v in pairs(effectOrders) do
			AddEffect(v)
		end
		effectOrders = {}

		ordersLocked = false
	end

	if clearFlag then
		this:DelObjVar("StatusEffects")
		this:DelModule("status_effect")
		return
	end

	--this:SystemMessage("Ready to start effect loop!")
	for k,v in pairs(effects) do
		if v.damage ~= nil then
			local damage = v.damage
			damage[2] = damage[2] / 1000 * EFFECT_TICK_SPEED
			this:SendMessage("ProcessDamage", damage) -- Apply damage
		elseif v.simpleDmg ~= nil then
			--this:SystemMessage("simpleDmg found")
			local dmgtarget = v.dmgTarget or "Chest" -- Default effect damage (poison) targets chest, heart is there
			local isCritical = v.isCritical or false
			local damage = v.simpleDmg / 1000 * EFFECT_TICK_SPEED
			--this:SystemMessage("Damage is "..damage)
			this:SendMessage("ProcessDamage", this, damage, "Poison", "TrueDamage", dmgTarget, isCritical)
		end

		if v.heal ~= nil then -- Simple healings
			local curHealth = this:GetCurHealth()
			local maxHealth = this:GetMaxHealth()
			local heal
			if curHealth + v.heal >= maxHealth then
				heal = maxHealth - curHealth
			else
				heal = v.heal
			end

			this:SetCurHealth(curHealth + heal)
		end

		local removeEffect = false
		if v.CustomTick ~= nil then
			--this:SendMessage("Applying CustomTick")
			removeEffect = v:CustomTick(effects, k, this) -- All effects, identifier of this effects, this
		end

		local handlers = ProduceEffectHandlers(this, v, "OnTick")
		for name,handler in pairs(handlers) do
			removeEffect = handler:OnTick()
		end

		effects[k].timeLeft = v.timeLeft - EFFECT_TICK_SPEED
		--this:SystemMessage("timeLeft "..effects[k].timeLeft)
		if effects[k].timeLeft <= EFFECT_TICK_SPEED then
			--this:SystemMessage("Ordering cure, we don't need another effect tick")
			removeEffect = true
		end

		if removeEffect then
			OrderCure(k, true)
			--DebugMessage("removeEffect == true; ordering cure")

			local handlers = ProduceEffectHandlers(this, v, "OnWearOff")
			for name,handler in pairs(handlers) do
				--DebugMessage("Calling handler "..name)
				removeEffect = handler:OnWearOff()
			end
		end
	end

	if needsTicking then
		if #effects < 1 then
			--this:SystemMessage("No ticking needed anymore, setting needsTicking to false")
			needsTicking = false
			return
		end

		this:ScheduleTimerDelay(TimeSpan.FromMilliseconds(EFFECT_TICK_SPEED), "StatusEffectTick")
		--this:SystemMessage("Poison tick end")
	end
end

-- Instantly removes effect. Never use directly, it is unsafe.
function RemoveEffect(id)
	local effect = effects[id]
	table.remove(effects, id)

	if effect.OnRemove ~= nil then effect:OnRemove(effects, this) end

	local handlers = ProduceEffectHandlers(this, effect, "OnRemove")
	for k,v in pairs(handlers) do
		v:OnRemove()
	end
end

-- Orders effect to be cured. Safe to use always.
function OrderCure(id,force)
	local force = force or false
	if effects[id] == nil then
		return "effectNotFound", "Effect with id "..id.." was not found. Outdated id?"
	end

	local allowCure = true
	if effects[id].AllowCure ~= nil and not force then
		allowCure = effects[id].AllowCure(id)
	end

	if allowCure then
		cureOrders[#cureOrders+1] = id
		return "success", "Curing effect before next tick."
	else
		return "prevented", "Something prevented curing effect, and force was false."
	end
end

-- Instantly adds effect. Never use directly, it is unsafe.
function AddEffect(effect)
	--this:SystemMessage("AddPoison called, count now is"..#effects)
	effects[#effects+1] = effect

	local handlers = ProduceEffectHandlers(this, effect, "OnApply")
	for k,v in pairs(handlers) do
		v:OnApply()
	end
end

function OrderEffect(effect,force)
	--this:SystemMessage("OrderEffect")
	local canApply = true
	if effect.CanApply ~= nil and not force then
		canApply = effect.CanApply(this) -- Param is this object
	end

	if not canApply then
		return "prevented", "Something prevented applying effect, and force was false."
	end

	effect.timeLeft = effect.duration
	effectOrders[#effectOrders+1] = effect

	return "success", "Applying effect before next tick."
end

-- Clears orders
function ClearOrders(group)
	if ordersLocked then return "lockedOrders", "Orders are currently locked." end
	ordersLocked = true

	local group = group or "all"

	if group == "cures" then cureOrders = {}
	elseif group == "effects" then effectOrders = {}
	else
		effectOrders = {}
		cureOrders = {}
	end

	ordersLocked = false
end

-- Wakes up effect ticking.
function WakeUp()
	if not needsTicking then
		--this:SystemMessage("No needsTicking, waking up...")
		this:ScheduleTimerDelay(TimeSpan.FromMilliseconds(EFFECT_TICK_SPEED), "StatusEffectTick")
		needsTicking = true
	end
end

-- Adds cure processing function, which will safely loop trought effects next tick
function AddCureProcess(effectName,removeCount)
	cureProcesses[#cureProcesses+1] = GetCureProcess(effectName, removeCount)
end

-- Internal cure process function
function ExecuteCureProcesses()
	--DebugMessage("Execute cure processes.")
	local curePending = {}
	for line,func in pairs(cureProcesses) do
		local toCure = func(effects)

		for k,v in pairs(toCure) do
			--DebugMessage("Cure "..k)
			curePending[k] = v
		end
	end

	for k,v in pairs(curePending) do
		OrderCure(k)
	end

	cureProcesses = {}
end

function ClearAll()
	clearFlag = true
end

RegisterEventHandler(EventType.Timer, "StatusEffectTick", ApplyEffectTick)

RegisterEventHandler(EventType.Message, "InflictEffect", OrderEffect)

RegisterEventHandler(EventType.Message, "CureEffect", OrderCure)

RegisterEventHandler(EventType.Message, "WakeUpEffects", WakeUp)

RegisterEventHandler(EventType.Message, "AddCureProcess", AddCureProcess)

RegisterEventHandler(EventType.Message, "ClearEffects", ClearAll)

RegisterEventHandler(EventType.Message, "HasDiedMessage", ClearAll)