require 'incl_PvPEvent'

-- the delay between spawn attempts, can be overridden by adding "spawnDelay" objvar
DEFAULT_DELAY_SECS = 120
-- the delay between spawn attempts, can be overridden by adding "spawnCount" objvar
DEFAULT_SPAWN_COUNT = 1



SpawnInfo = 
{
    Necromancer = {
        { ["Count"] = 1, ["TemplateId"] = "vs_pvp_avatar_necromancer" },
    },
    Paladin = {
        { ["Count"] = 1, ["TemplateId"] = "vs_pvp_avatar_paladin" },
    },
    Witches = {
        { ["Count"] = 1, ["TemplateId"] = "vs_pvp_avatar_witch" },
    Slayers = {
        { ["Count"] = 1, ["TemplateId"] = "vs_pvp_avatar_slayer" },
    },
}



function CheckSpawn()	
    local team = this:GetObjVar("MobileTeamType")
    local spawnCount = SpawnInfo[team][1].Count or DEFAULT_SPAWN_COUNT
    local templateId = SpawnInfo[team][1].TemplateId

    if( templateId == nil ) then
        DebugMessage("[simple_mob_spawner::CheckSpawn] No template id specified.")
        return
    end

    local spawnData = this:GetObjVar("spawnData")      
    -- NOTE: If spawn count is made smaller, we are not destroying the extra mobs 
    if spawnData == nil then
        spawnData = {}
        this:SetObjVar("spawnData", spawnData)
    elseif #spawnData > spawnCount then
        for i=spawnCount+1,#spawnData do
            table.remove(spawnData,i)
        end
    end    

    --DebugMessage("---CheckSpawn SpawnCount: "..spawnCount)     

    for i=1, spawnCount do
        --DebugMessage("---Checking i="..i)     
        if( spawnData[i] == nil or not(spawnData[i]:IsValid()) ) then
            --DebugMessage("---Create "..templateId)     
            -- only create one at a time
            CreateObj(templateId, this:GetLoc(), "mobSpawned", i)
            break
        end
    end
end

function HandleStartRespawnTimerMessage()
    if( GetPvPGameState() ~= "Match" ) then
        this:ScheduleTimerDelay(TimeSpan.FromSeconds(1), "StartRespawnTimer")
    else
        local spawnDelay = this:GetObjVar("spawnDelay") or DEFAULT_DELAY_SECS
        this:ScheduleTimerDelay(TimeSpan.FromSeconds(spawnDelay), "SpawnTimer")
        this:ScheduleTimerDelay(TimeSpan.FromSeconds(spawnDelay-30), "SpawnAlertTimer")
    end
end

RegisterEventHandler(EventType.Message,"reset",
    function ()
        this:RemoveTimer("SpawnTimer")
        this:RemoveTimer("SpawnAlertTimer")
        this:ScheduleTimerDelay(TimeSpan.FromSeconds(1), "StartRespawnTimer")
    end)

RegisterEventHandler(EventType.Message, "StartRespawnTimer", 
    function()
        HandleStartRespawnTimerMessage()
    end)

RegisterEventHandler(EventType.Timer, "StartRespawnTimer", 
    function()
        HandleStartRespawnTimerMessage()
    end)

RegisterEventHandler(EventType.ModuleAttached, "vs_avatar_respawner", 
    function()
        HandleStartRespawnTimerMessage()
    end)

RegisterEventHandler(EventType.CreatedObject, "mobSpawned", 
    function(success, objref, index)
        if( success) then
            local spawnData = this:GetObjVar("spawnData") 
            spawnData[index] = objref
            this:SetObjVar("spawnData",spawnData)
            PvPBroadcast("[3D91FF]The [-][FFF544]" .. this:GetObjVar("MobileTeamType") .. "[-][3D91FF] avatar has respawned.[-]")
            objref:SetFacing(this:GetFacing())
            objref:SetObjVar("Respawner", this)
        end
    end)

RegisterEventHandler(EventType.Timer, "SpawnTimer", 
    function()
        CheckSpawn()
    end)

RegisterEventHandler(EventType.Timer, "SpawnAlertTimer", 
    function()
        PvPBroadcast("[3D91FF]The [-][FFF544]" .. this:GetObjVar("MobileTeamType") .. "[-][3D91FF] avatar will respawn in [-][FFF544]30 seconds[-][3D91FF].[-]")
    end)