--[[
File: global_utils.lua
Various utility Functions
~Yorlik
--]]

-- mimicks the C printf function
function printf (s,...)
           return print(s:format(...))
         end -- function

-- mimicks the C printf function with Debug additions
function DebugMessageF (s,...)
           return DebugMessage(s:format(...))
         end -- function

-- Conditonal DebugMessages
-- DebugMessage only for Players ...
function DebugPlayerMessage(...)
    if this~=nil and this:IsPlayer() then
        DebugMessage(...)
    end
end
-- DebugMessageF only for Players ...
function DebugPlayerMessageF(s,...)
    if this~=nil and this:IsPlayer() then
        DebugMessageF(s,...)
    end
end


-- non shallow simple table deepcopy
function deepcopy(orig)
    local orig_type = type(orig)
    local copy
    if orig_type == 'table' then
        copy = {}
        for orig_key, orig_value in next, orig, nil do
            copy[deepcopy(orig_key)] = deepcopy(orig_value)
        end
        setmetatable(copy, deepcopy(getmetatable(orig)))
    else -- number, string, boolean, etc
        copy = orig
    end
    return copy
end         

-- Source: http://lua-users.org/wiki/MakingLuaLikePhp
-- Credit: http://richard.warburton.it/
function explode(div,str)
    if (div=='') then return false end
    local pos,arr = 0,{}
    for st,sp in function() return string.find(str,div,pos,true) end do
        table.insert(arr,string.sub(str,pos,st-1))
        pos = sp + 1
    end
    table.insert(arr,string.sub(str,pos))
    return arr
end

-- from https://www.lua.org/pil/19.3.html
function pairsByKeys (t, f)
  local a = {}
  for n in pairs(t) do table.insert(a, n) end
  table.sort(a, f)
  local i = 0      -- iterator variable

  local iter = function ()   -- iterator function
    i = i + 1
    if a[i] == nil then return nil
    else return a[i], t[a[i]]
    end
  end

  return iter
end